#ifndef HARDWIRE_H_
#define HARDWIRE_H_

#include "rxduino.h"
#include "iodefine_gcc63n.h"

#define BUFFER_LENGTH  32

#define STANDARD_MODE   0
#define FAST_MODE       1
#define FAST_MODE_PLUS  2

#define CHAR    1
#define SHOAT   2
#define INT     4

#define BYTE1 1
#define BYTE2 2
#define BYTE3 3
#define BYTE4 4

#ifdef __cplusplus
	extern "C" {
#endif

class TwoHardWire {
  private:
    static uint8_t txAddress;
    static uint8_t txBuffer[];
    static uint8_t txBufferLength;
    static uint8_t txBufferIndex;
    
    static uint8_t rxBuffer[];
    static uint8_t rxBufferLength;
    static uint8_t rxBufferIndex;
    
  public:
    TwoHardWire();
    void begin(void);
    void begin(uint8_t, uint8_t, uint8_t);
    void Mode(uint8_t);
    void beginTransmission(uint8_t);
    uint8_t endTransmission(void);
    boolean write(uint8_t);
    uint8_t write(const uint8_t*, uint8_t);
    uint8_t write(const uint32_t*, uint8_t, uint8_t);
    uint8_t write(const uint8_t, uint8_t);
    uint8_t write(const uint32_t, uint8_t, uint8_t);
    int requestFrom(int read_add, int quantity);
    int requestFrom(uint8_t, uint8_t);
    uint8_t read(void);
    uint8_t available(void);
    
    void onReceive0( void (*)(int) );
    void onReceive1( void (*)(int) );
    void onReceive2( void (*)(int) );
    void onRequest0( void (*)(void));
    void onRequest1( void (*)(void));
    void onRequest2( void (*)(void));
    
    static void (*user_onRequest0)(void);
    static void (*user_onRequest1)(void);
    static void (*user_onRequest2)(void);
    static void (*user_onReceive0)(int);
    static void (*user_onReceive1)(int);
    static void (*user_onReceive2)(int);
    
    inline size_t write(unsigned long n) { return write((uint8_t)n); }
    inline size_t write(long n) { return write((uint8_t)n); }
    inline size_t write(unsigned int n) { return write((uint8_t)n); }
    inline size_t write(int n) { return write((uint8_t)n); }
    
};

extern TwoHardWire hardWire;

#ifdef __cplusplus
	}
#endif

#endif

